package org.bitbucket.bradleysmithllc.etlunit.test_support;

import org.bitbucket.bradleysmithllc.etlunit.ProcessDescription;
import org.bitbucket.bradleysmithllc.etlunit.ProcessExecutor;
import org.bitbucket.bradleysmithllc.etlunit.ProcessFacade;

import java.io.*;

public class MockProcessExecutor implements ProcessExecutor
{
	public MockProcessExecutor(ProcessCallback processCallback)
	{
		this.processCallback = processCallback;
	}

	public static interface ProcessCallback
	{
		public MockProcessFacade mock(ProcessDescription pd) throws IOException;
	}

	public static interface MockProcessFacade
	{
		void mockWaitForProcess();

		int mockCompletionCode();

		void mockKill();

		void mockWaitForStreams();

		Writer getWriter();

		BufferedReader getReader();

		BufferedReader getErrorReader();

		StringBuffer getInputBuffer();

		StringBuffer getErrorBuffer();

		OutputStream getOutputStream();

		InputStream getInputStream();

		InputStream getErrorStream();
	}

	private final ProcessCallback processCallback;

	@Override
	public ProcessFacade execute(final ProcessDescription pd) throws IOException
	{
		final MockProcessFacade mpf = processCallback.mock(pd);

		return new ProcessFacade()
		{
			@Override
			public ProcessDescription getDescriptor()
			{
				return pd;
			}

			@Override
			public void waitForCompletion()
			{
				mpf.mockWaitForProcess();
			}

			@Override
			public int getCompletionCode()
			{
				return mpf.mockCompletionCode();
			}

			@Override
			public void kill()
			{
				mpf.mockKill();
			}

			@Override
			public void waitForStreams()
			{
				mpf.mockWaitForStreams();
			}

			@Override
			public void waitForOutputStreamsToComplete() {
			}

			@Override
			public Writer getWriter()
			{
				return mpf.getWriter();
			}

			@Override
			public BufferedReader getReader()
			{
				return mpf.getReader();
			}

			@Override
			public BufferedReader getErrorReader()
			{
				return mpf.getErrorReader();
			}

			@Override
			public StringBuffer getInputBuffered() throws IOException
			{
				return mpf.getInputBuffer();
			}

			@Override
			public StringBuffer getErrorBuffered() throws IOException
			{
				return mpf.getErrorBuffer();
			}

			@Override
			public OutputStream getOutputStream()
			{
				return mpf.getOutputStream();
			}

			@Override
			public InputStream getInputStream()
			{
				return mpf.getInputStream();
			}

			@Override
			public InputStream getErrorStream()
			{
				return mpf.getErrorStream();
			}
		};
	}
}